#!/bin/bash -e
#SCRIPT_PURPOSE: make commands run from within a docker container

PYVERS="python2.7"
# Generate clean log messages from this part of the build.
# Prefix with Docker Make for clarity.
function fxnPrintMsg ()
{
	echo ""
	echo "DM: ________________________________"
	echo "DM:"
	echo "DM: $1"
	echo "DM:________________________________"
	echo ""
}

# Clean the build area
function fxnClean()
{
	fxnPrintMsg "Cleaning from [ $(pwd) ]"
	
	$PYVERS netq_backend/setup.py clean
	git clean -dxf
	rm -rf build __pycache__
	rm -rf *.egg-info
}
#
# This build code gets run in the container
#
function fxnBuildInContainer()
{
	CURDIR="$(pwd)"
	fxnPrintMsg "Building from [ $CURDIR ]"

	fxnPrintMsg "Apt-get installing build dependencies."
	# clbuilder can sudo without password. If the whole script
	# is run as root the NPM download creates root owned
	# leftovers that don't want to clean out.

	fxnPrintMsg "Running apt-get update"
	sudo apt-get update

	fxnPrintMsg "Running apt-get upgrade"
	sudo apt-get upgrade -y

	fxnPrintMsg "Installing build dependencies"
	sudo mk-build-deps --install ./debian/control --tool "apt-get -y"

	fxnPrintMsg "Python build"
	python2.7 netq_backend/setup.py build 

	fxnPrintMsg "Yarn install."
	/usr/bin/yarn install

	fxnPrintMsg "Yarn build."
	/usr/bin/yarn run build

	#
	# .git files need to be left as there are later checkouts.
	# however, if sbuild tries to do a git 3.0 build it becomes aware of the
	# subterfuge and fails to find files around dists/*
	#
	fxnPrintMsg "Setting debian/source/format to 1.0 for packaging."
	echo "1.0" > ${CURDIR}/debian/source/format
	echo "" >>   ${CURDIR}/debian/source/format

	fxnPrintMsg "Done Docker build."	
}

#
# MAIN
#

# Put in more robust parsing when the complexity warrants it...
case $1 in
	"--clean" )
		fxnClean
		;;

	"--build" )
		fxnBuildInContainer
		;;

	* )
		echo "Unknown parameter [ $1 ]. Options are '--clean' or '--build'"
		;;

esac



