import cytoscape from 'cytoscape';
// TODO: put traceData into vuex
import traceData from 'data/trace.json';

// TODO: get this from user input (once we have it)
const DESTINATION_ID = "45.0.0.24";

// work on a copy of paths
let paths = traceData.paths.slice(0);

let nodes = {};
let edges = {};

const mungePath = function(_path, pathIndex) {
    // skip hops that we don't need
    // TODO: can we safely ignore these?
    let path = _path.filter(function(hop) {
        // if it has a 'from' prop, we don't want it
        return !hop.from;
    })
    // TODO: extract data (ip?) from first element?
    // TODO: check for loops?
    // HACK: add a bogus hop to the end to make sure all the edges get added
    //       right
    path[path.length] = {
        node: DESTINATION_ID,
    };

    // TODO: is "hop" a good name for this?
    let lastHop = null;
    let curHop = null;
    for (let i = 0; i < path.length; i++) {
        curHop = path[i];
        let nodeName = curHop.node;
        // FIXME: messy/hacky
        let nodeData = {
            id: nodeName,
            // HACK
            paths: [],
            classArray: [],
        };
        if (!nodes[nodeName]) {
            nodes[nodeName] = {
                // HACK
                data: nodeData,
            };
        }
        nodes[nodeName].data.paths.push(pathIndex);
        nodes[nodeName].data.classArray.push(`path-${pathIndex}`);

        // extract edge
        if (lastHop) {
            let source = lastHop.node;
            let target = curHop.node;
            // NOTE: this can't be id because nexthop names can be duplicates
            let edgeName = lastHop.nexthop;
            // NOTE: this will become the id
            let edgeProp = `${source}:${edgeName}--${target}`;
            // TODO: can the contents of an edge item be different if I use this
            //       edgeProp?
            // FIXME: messy/hacky
            let edgeData = {
                source,
                target,
                // TODO: is there a better place for this than data?
                label: edgeName,
                id: edgeProp,
                paths: [],
                classArray: [],
            };
            if (!edges[edgeProp]) {
                edges[edgeProp] = {
                    // HACK
                    data: edgeData,
                };
            }
            // HACK: pls make it stop
            edges[edgeProp].data.paths.push(pathIndex);
            edges[edgeProp].data.classArray.push(`path-${pathIndex}`);
        }
        lastHop = curHop;
    }
}

paths.forEach(mungePath);

let nodeElements = Object.keys(nodes).map(function(prop) {
    let element = nodes[prop];
    element.classes = element.data.classArray.join(' ');
    return element;
});
let edgeElements = Object.keys(edges).map(function(prop) {
    let element = edges[prop];
    element.classes = element.data.classArray.join(' ');
    return element;
});
let elements = [
    ...nodeElements,
    ...edgeElements,
];

let rows = traceData.totalPaths;

const COLOR_DEFAULT = '#ccc';
const COLOR_SELECTED = '#080';
const COLOR_HOVER = '#777';
const COLOR_HOVER_AND_SELECTED = '#040';

const STYLE_SELECTED = {
    'background-color': COLOR_SELECTED,
    'line-color': COLOR_SELECTED,
    'target-arrow-color': COLOR_SELECTED,
};
const STYLE_HOVER = {
    'background-color': COLOR_HOVER,
    'line-color': COLOR_HOVER,
    'target-arrow-color': COLOR_HOVER,
};
const STYLE_HOVER_AND_SELECTED = {
    'background-color': COLOR_HOVER_AND_SELECTED,
    'line-color': COLOR_HOVER_AND_SELECTED,
    'target-arrow-color': COLOR_HOVER_AND_SELECTED,
};

const CLASS_PATH_SELECTED = 'path-selected';
const CLASS_PATH_HOVER = 'path-hover';
export default {
    mounted() {
        let cy = cytoscape({
            container: this.$el,
            elements,
            style: [
                {
                    selector: 'node',
                    style: {
                        'background-color': COLOR_DEFAULT,
                        'label': 'data(id)',
                    },
                },
                {
                    selector: 'edge',
                    style: {
                        'curve-style': 'bezier',
                        'width': 3,
                        'source-label': 'data(label)',
                        'source-text-offset': '40',
                        'line-color': COLOR_DEFAULT,
                        'target-arrow-color': COLOR_DEFAULT,
                        'target-arrow-shape': 'triangle',
                    },
                },
                {
                    selector: `.${CLASS_PATH_SELECTED}`,
                    style: STYLE_SELECTED,
                },
                {
                    selector: `.${CLASS_PATH_HOVER}`,
                    style: STYLE_HOVER,
                },
                {
                    selector: `.${CLASS_PATH_SELECTED}.${CLASS_PATH_HOVER}`,
                    style: STYLE_HOVER_AND_SELECTED,
                },
            ],
            layout: {
                // TODO: write your own layout plugin
                // TODO: figure out a sane way to do this
                name: 'breadthfirst',
                roots: [ 'spine-1' ],
                transform(node, position) {
                    // flip x and y
                    let x = position.y;
                    let y = position.x;
                    return { x, y };
                },
            },
        });
        // FIXME: duplicate code
        // TODO: make sure classArray is present
        cy.on('select', function(event) {
            let { target } = event;
            // early exit on invalid targets
            if (typeof target.data !== 'function') {
                return;
            }
            target.data().classArray.forEach(function(className) {
                let selector = `.${className}`;
                let coll = cy.collection(selector);
                coll.addClass(CLASS_PATH_SELECTED);
            });
        });
        cy.on('unselect', function(event) {
            let { target } = event;
            // early exit on invalid targets
            if (typeof target.data !== 'function') {
                return;
            }
            target.data().classArray.forEach(function(className) {
                let selector = `.${className}`;
                let coll = cy.collection(selector);
                coll.removeClass(CLASS_PATH_SELECTED);
            });
        });
        cy.on('mouseover', function(event) {
            let { target } = event;
            // early exit on invalid targets
            if (typeof target.data !== 'function') {
                return;
            }
            target.data().classArray.forEach(function(className) {
                let selector = `.${className}`;
                let coll = cy.collection(selector);
                coll.addClass(CLASS_PATH_HOVER);
            });
        });
        cy.on('mouseout', function(event) {
            let { target } = event;
            // early exit on invalid targets
            if (typeof target.data !== 'function') {
                return;
            }
            target.data().classArray.forEach(function(className) {
                let selector = `.${className}`;
                let coll = cy.collection(selector);
                coll.removeClass(CLASS_PATH_HOVER);
            });
        });
    },
};
