import {
    ACTION_FETCH_EVENT_MESSAGES,
    ACTION_FETCH_EVENT_COUNTS,
} from 'store/action-types.js';
import {
    EVENT_MESSAGES_UPDATE,
    EVENT_COUNTS_UPDATE,
} from 'store/mutation-types.js';

import { format } from 'date-fns';

const API = (function(deps) {
    const URL_API = '/webapi/v1';
    const URL_EVENT = `${URL_API}/event`;
    const URL_EVENT_MESSAGES = `${URL_EVENT}/messages`;
    const URL_EVENT_COUNTS = `${URL_EVENT}/counts`;
    // headers common across all API requests
    const HEADERS_COMMON = new Headers({
        'Content-Type': 'application/json',
    });

    // TODO: define the rest of these somewhere, I guess
    const METHOD_POST = 'POST';

    return {
        async fetchEventMessages(options={}) {
            let {
                startTime,
                endTime,
                filters={},
                // TODO: filters
            } = options;

            let requestBody = {
                // TODO:RJM: is this a reasonable start time
                "startTime": startTime,
                // TODO:RJM: is this a reasonable end time
                "endTime": endTime,
                "filters": filters,
            };
            let response = await fetch(URL_EVENT_MESSAGES, {
                method: METHOD_POST,
                headers: HEADERS_COMMON,
                body: JSON.stringify(requestBody),
            });

            return await response.json();
        },
        async fetchEventCounts(options={}) {
            let {
                startTime,
                endTime,
                // TODO: make this automatic
                // TODO: use prespecified bucket values
                timeInterval,
                filters={},
            } = options;

            let requestBody = {
                startTime,
                endTime,
                timeInterval,
                filters,
            };

            let response = await fetch(URL_EVENT_COUNTS, {
                method: METHOD_POST,
                headers: HEADERS_COMMON,
                body: JSON.stringify(requestBody),
            });

            return await response.json();
        },
    };
})();

// TODO: these guys need a home
// NOTE: options shouldn't change; data will
let options = {
    // keys from fetched JSON structure
    inKeys: [
        // NOTE: order here determines order of columns
        'timestamp',
        'source',
        'eventType',
        // TODO: mechanism for mapping numerical value to severity
        'severity',
        'message',
    ],
    // mappings to displayable keys/labels
    columnLabelMappings: {
        'timestamp': 'Time',
        'severity': 'Severity',
        'eventType': 'Type',
        'source': 'Source',
        'message': 'Message',
    },
    displayTransformers: {
        // TODO:RJM: do I want to use the inKey or the columnKey?
        timestamp(value) {
            // timestamps from server are in seconds
            let date = new Date(value * 1000);
            // TODO: extract magic string
            return format(date, 'MMM D, YYYY HH:mm:ss');
        },
    },
};

// TODO: Better name!
const eventMessagesMungeData = function(rawData, options) {
    //  TODO: move `options` into this function
    let {
        displayTransformers,
        inKeys,
        columnLabelMappings,
    } = options;

    // NOTE: this transforms the cells
    let data = rawData.eventMessages.map(function(tableRow) {
        let resultRow = {};
        inKeys.forEach(function(columnKey) {
            let value = tableRow[columnKey];

            let displayValue = value;
            if (displayTransformers[columnKey]) {
                displayValue = displayTransformers[columnKey](value);
            }
            // HACK: probably really inefficient too
            let outKey = columnLabelMappings[columnKey];
            resultRow[outKey] = {
                value,
                displayValue,
            };
        });
        return resultRow;
    });

    return data;
};

// TODO: Better name!
const eventCountsMungeData = function(rawData) {
    // TODO: do I need to anything else here?
    let data = rawData.data;
    return data;
};

const actions = {
    async [ACTION_FETCH_EVENT_MESSAGES](context) {
        let startTime = context.state.startTime;
        let endTime = context.state.endTime;
        // TODO: filters
        let response = await API.fetchEventMessages({
            startTime,
            endTime,
        });
        let eventMessages = eventMessagesMungeData(response, options);
        context.commit(EVENT_MESSAGES_UPDATE, eventMessages);
    },
    async [ACTION_FETCH_EVENT_COUNTS](context) {
        let startTime = context.state.startTime;
        let endTime = context.state.endTime;
        // HACK: hardcoded time interval
        // TODO: calculate time interval
        let timeInterval = 15;
        // TODO: filters
        let response = await API.fetchEventCounts({
            startTime,
            endTime,
            timeInterval,
        });
        // TODO: rename?
        let eventCounts = eventCountsMungeData(response);
        context.commit(EVENT_COUNTS_UPDATE, eventCounts);
    },
};

export default actions;
