const {
    HotModuleReplacementPlugin,
    NamedModulesPlugin,
} = require('webpack');
const merge = require('webpack-merge');
const common = require('./webpack.common.js');

const fs = require('fs');

// try to load secrets file
let secrets;
try {
    secrets = JSON.parse(fs.readFileSync('./secrets.json'));
} catch(e) {
    console.error('WARNING: secrets.json failed to load');
    secrets = {};
}


let devServer = {
    contentBase: './dist',
    hot: true,
};
// only set up proxy if a target is specified
if (secrets.devProxyTarget) {
    devServer.proxy = {
        '/webapi': {
            target: secrets.devProxyTarget,
            secure: false,
        },
    };
}

module.exports = merge(common, {
    devtool: 'cheap-module-eval-source-map',
    devServer,
    plugins: [
        new HotModuleReplacementPlugin(),
        new NamedModulesPlugin(),
    ],
});
